@@ -6,7 +6,7 @@ package Mite::Class;
 use Mite::Miteception;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.001003';
+our $VERSION   = '0.001004';
 
 use Path::Tiny;
 use mro;
@@ -272,8 +272,15 @@ sub _compile_bless {
 
 sub _compile_strict_constructor {
     my ( $self, $classvar, $selfvar, $argvar, $metavar ) = @_;
-    local $Type::Tiny::AvoidCallbacks = 1;
-    my $check = Enum->of( keys %{ $self->all_attributes } )->inline_check( '$_' );
+
+    my @allowed =
+        grep { defined $_ }
+        map { $_->init_arg }
+        values %{ $self->all_attributes };
+    my $check = do {
+        local $Type::Tiny::AvoidCallbacks = 1;
+        Enum->of( @allowed )->inline_check( '$_' );
+    };
 
     return sprintf 'my @unknown = grep not( %s ), keys %%{%s}; @unknown and require Carp and Carp::croak("Unexpected keys in constructor: " . join(q[, ], sort @unknown));',
         $check, $argvar;
